#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <poll.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include "linux/mcst/mpv_io.h"

#define        FNC     __FUNCTION__

char	*name_prog = NULL;
char	name_bus[128];
char	*name_dev = NULL;
int	debug = 0;
int	silent = 0;
int	verbose = 0;
int	prn_instr = 0;
int	inst = 0;
int	use_out = 0; /* 1 - использовать шины OUT вместо ST */
int	send_intr = 0; /* 1 - на шине OUT посылать импульс */
int	st = 8;	/* N ST - по умолчанию */
int	state = 1;	/* значение ST - по умолчанию 1 */

int	Npulses	= 1;		// число импульсов
int	time_interval = 0;	// интервал между импульсами (милисек)

char my_dir_name[2048];
char *my_prog_name = NULL;
int
set_current_dir (char *prog) {
        int     i, n, res = 0, ret;

        if (!realpath(prog, my_dir_name)) {
            res = 1;
            printf("%s Не удалось получить realpath от %s, res = NULL - '%m'\n",
                FNC, prog);
        }
        if (verbose > 1)
    	    printf("Имя программы =%s, Полный путь к программе =%s (%d байт)\n",
                prog, my_dir_name, strlen(my_dir_name));

        for (n = 0, i = strlen(my_dir_name); i > 0; i--) {
    	    if (my_dir_name[i] == '/') {
                if (++n == 1)
            	    my_prog_name = &(my_dir_name[i +1]);
                my_dir_name[i] = 0;
                break;
    	    }
        }

        if (verbose)
    	    printf("Каталог теста =%s, имя программы =%s\n",
                my_dir_name, my_prog_name);
        ret = chdir(my_dir_name);
        if (ret) {
    	    printf("%s chdir(%s), рез. =%d %s\n",
                FNC, my_dir_name, ret, strerror(errno));
            res = 1;
        }

        return (res);
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ set_current_dir


static int
do_st (int fd, int state) {
	int	res1 = 0;
	int	res2 = 0;
	if (state > 3) {
	    if (state == 4) {
		if (res1 = do_st(fd, 1) == 0 || debug)
		    res2 = do_st(fd, 0);
	    } else {
		if (res1 = do_st(fd, 0) == 0 || debug)
		    res2 = do_st(fd, 1);
	    }
	    return (res1 || res1);
	}
	if (ioctl(fd, MPVIO_SET_STATE, state) != 0) {
	    printf("%s :Failed ioctl(MPVIO_SET_STATE, %d) - %m\n", FNC);
	    return -1;
	}
	if (verbose)
	    printf("%s :Выполнен SET_STATE =%d для %s\n", FNC, state, name_bus);
	return 0;
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ do_st

static void
do_out (int fd) {
	if (ioctl(fd, MPVIO_SEND_INTR, 0) != 0) {
	    printf("%s :Failed ioctl(MPVIO_SEND_INTR) - %m\n", FNC);
	    return;
	}
	if (verbose)
	    printf("%s :Выполнен SEND_INTR для %s\n", FNC, name_bus);
}

static void
print_manual (int res) {
    printf(
"\n   Программа формирования состояния шин STATE или OUT.\n"
"Используется :\n"
"    - для установки состояния шин модулей mpv;\n"
"    - для теста MPV4, при проверке модуля MPV/P;\n"
"    - для отладки \"Программы Контроля состояния температурного датчика ВК УВК/С-А\";\n"
"    - для формирования импульсного сигнал или серии испульсов.\n"
"\n"
"Параметры программы %s :\n"
"        -S - молчаливый режим;\n"
"        -i<N>    N экземпляра mpv (по умолчанию =0);\n"
"        -s<N_ST> [<значение ST>]\n"
"                N ST (по умолчанию =8), значение ST (по умолчанию =1)\n"
"                При этом в зависимости от значения ST =\n"
"                  -0 - на шине устанавливается сигнал =0,\n"
"                  -1 - на шине устанавливается сигнал =1,\n"
"                  -2 - формируется положительный импульсный сигнал,\n"
"                  -3 - формируется отрицательный импульсный сигнал,\n"
"                  -4 - формируется положительный псевдо-импульсный сигнал,\n"
"                  -5 - формируется отрицательный псевдо-импульсный сигнал\n"
"                  (псевдо-импульсный сигнал формируется 2-мя командами ioctl);\n"
"        -n <количество> - число испульсов в серии испульсов;\n"
"        -t <время милисек> - интервал между импульсами (по умолчанию =0);\n"
"        -o - использовать шины OUT вместо шин STATE;\n"
"        -I - на шине OUT формируется положительный импульсный сигнал\n"
"                        используя ioctl MPVIO_SEND_INTR.\n"
"\n"
"   Вспомогательный режим (используется из скрипта pre_set_state) :\n"
"        -D <спец.файл к шине mpv> 0|1\n"
"\n"
"    Отладочные опции :\n"
"        -h - печать данной инструкции;\n"
"        -v - вкл.отладочную печать;\n"
"        -D - отладочный режим\n"
"\n",
	name_prog);
exit (res == 1);
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ print_manual

static void
parse_arg (int ac, char ** av) {
	int	i;
	int	err = 0;

	name_prog = av[0];
	while ((i = getopt(ac, av, "hDd:vi:n:Ios:St:")) != EOF) {
		switch (i) {
		case	'h':
			prn_instr = 1;
			break;
		case	'i':
			inst = atoi(optarg);
			break;
		case	'n':
			Npulses = atoi(optarg);
			break;
		case	't':
			time_interval = atoi(optarg);
			break;
		case	'd':
			name_dev = optarg;
			optarg = av[optind++];
			if (optind > ac || *optarg == '-') {
			    printf("\t%s !!! -d %s !!! где state ???\n",
				FNC, name_dev);
			    exit (-1);
			}
			else
			    state = atoi(optarg);
			    if (!isdigit(optarg[0])) {
				printf("\t%s !!! -d %s %s !!! где state ???\n",
				    FNC, name_dev, optarg);
				exit (-1);
			    }
			if (state < 0 || state > 1) {
				printf("\t%s !!! -d %s %s !!! state неверный\n",
				    FNC, name_dev, optarg);
				exit (-1);
			}
			break;
		case	'D':
			debug++;
			break;
		case	'v':
			verbose++;
			break;
		case	'S':
			silent++;
			break;
		case	's':	/* N st */
			st = atoi(optarg);
			optarg = av[optind++];
			if (optind > ac || *optarg == '-') {
			    optind--; break;
			}
			else
			    state = atoi(optarg);
			break;
		case	'o':	/* использовать шины OUT вместо ST */
			use_out = 1;
			break;
		case	'I':	/* на шине OUT посылать импульс */
			send_intr = 1;
			break;
		case	'?':
			prn_instr = 1;
			err = 1;
			break;
		}
	}
	if (prn_instr)
	    print_manual(err);
	if (verbose)
	    silent = 0;
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ parse_arg

/*
 *   Для режима, когда указан state > 1 - т.е. при выдачи импульса,
 * значение на заданной шине должно быть ранее правильно установленно.
 * Значение шины должно быть :
 *	для формирования положительного импульса == 0;
 *	для формирования отрицательного импульса == 1;
 */
void
pre_set_state (void) {
    int	res, i;
    char cmd[256], vrb[64] = { 0 };

    if (verbose)
	sprintf(vrb, "export VRB=%d; ", verbose);

    if (send_intr) {
    char *name;
	for (i = 0; name_bus[i]; i++)
	    if (name_bus[i] == '/')
		name = &name_bus[i+1];
	sprintf(cmd, "%secho 0 > %s", vrb, name);
    }
    else
	sprintf(cmd, "%sbash pre_set_state %s %d", vrb, name_bus, state % 2);

    if (verbose)
	printf("%s :system(%s)\n", FNC, cmd);
    res = system(cmd);
    if (verbose)
	printf("%s :res system(%s) =%d\n", FNC, cmd, res);
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ pre_set_state

void
form_msg_start (void) {
	char	M_[128] = {0}, *M = (char *)M_;
    if (silent)
	return;

    if (send_intr)
	M += sprintf(M, "%s :выдача ", name_prog);
    else
	M += sprintf(M, "%s :формирование ", name_prog);

    if (Npulses > 1) {
	M += sprintf(M, "серии из %d ", Npulses);
	if (time_interval)
	    M += sprintf(M, "(с интервалом =%d милисек) ", time_interval);
    }
    if (send_intr)
	M += sprintf(M, "импульс%s ", Npulses > 1 ? "ов" : "а");
    else if (state > 1)
	M += sprintf(M, "%s %sимпульс%s ",
		state % 2 ? "положительных" : "отрицательных",
		state > 3 ? "псевдо-" : "",
		Npulses > 1 ? "ов" : "а");
    else
	M += sprintf(M, "уровня =%d ", state);
    printf("%sна шине =%s\n", M_, name_bus);
} // END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ form_msg_start

#include <time.h>
void
sleep_msec (int time) {
static  struct timespec slp_time;
        slp_time.tv_sec = time/1000;
        slp_time.tv_nsec = (time%1000) *1000000;
        nanosleep(&slp_time, NULL);
}

int
main (int argc, char *argv[]) {
	int	fd, N;

	parse_arg(argc, argv);
	set_current_dir(argv[0]);
	if (name_dev)
	    sprintf(name_bus, "%s", name_dev);
	else if (use_out)
	    sprintf(name_bus, "/dev/mpv_%d_out:%d", inst, st);
	else
	    sprintf(name_bus, "/dev/mpv_%d_st:%d", inst, st);

	form_msg_start();
	fd = open(name_bus, O_RDWR);
	if (fd < 0) {
	    printf("Failed open(%s): %m\n", name_bus);
	    if (debug)
		;
	    else
		return (-1);
	}
	pre_set_state();
	for (N = 1; N++ <= Npulses; ) {
	    if (send_intr)
		do_out(fd);
	    else
		do_st(fd, state);
	    if (time_interval)
//		usleep(time_interval);
		sleep_msec(time_interval);
	}
	(void) close(fd);

	return (0);
}
