#!/bin/bash

# set_route :
#   ethernet setup for fake loopback interface with NAT usage
#   Let ETH_1 and ETH_2 be the two ethernet cards;
#   IP_1 and IP_2 its IP address;
#   MAC1 and MAC2 its MAC address respectively.
#   We will be using two dummy IP addresses: IPw_1 and IPw_2.

TRIPLEPINGFILE="/tmp/eth_iperf.tripleping"
CONFIG_FILE="/tmp/eth_iperf.conf"

PSTP="/opt/mcst/check/STP/bin"
if [[ ! -d ${PSTP} ]]; then
	PSTP="/opt/mcst/STP/bin"
fi

umask 002

TESTTIME=60
PARALLEL=0
READCONFFILE=

IPERF=`command -v iperf`
IARGS="-i 1"
if [[ -z $IPERF ]]; then
	IPERF=`command -v iperf3`
fi
if [[ -z $IPERF ]]; then
	echo
	echo "ERROR !"
	echo "You must install iperf or iperf3 !"
	echo
	exit 1
fi

CMDPING="ping -W 1 -c 1 "

TMP="/tmp/STP"
[[ -d ${TMP} ]] || mkdir -m 0755 ${TMP}
IPTABLES="${TMP}/iptables.cfg"
IPROUTES="${TMP}/iproutes.cfg"
ARP="${TMP}/arp.cfg"
MSG="${TMP}/msg-err"

print_date() {
    d=`date`
    echo "Current date: $d"
    [[ ! -z $1 ]] && echo "Finish  date: `date -d @$1`"
    echo
}

set_route ()
{
	((N_in_set_route++))
	((VRB)) && echo -e "\nset_route $*"
	ETH_1=$1; IP_1=$2; MAC1=$3; IPw_1=$4
	ETH_2=$5; IP_2=$6; MAC2=$7; IPw_2=$8
	{
	ip route add $IPw_1 dev $ETH_2
	ip route add $IPw_2 dev $ETH_1

	iptables -t nat -A POSTROUTING -d $IPw_2 -j SNAT --to-source $IPw_1
	iptables -t nat -A POSTROUTING -d $IPw_1 -j SNAT --to-source $IPw_2
	iptables -t nat -A PREROUTING -d $IPw_1 -j DNAT --to-destination $IP_1
	iptables -t nat -A PREROUTING -d $IPw_2 -j DNAT --to-destination $IP_2

	arp -s $IPw_1 $MAC1
	arp -s $IPw_2 $MAC2
	} 2>&1 | grep "initialize iptables table" > ${MSG}
} # END ~~~~~~~~~~~~~~~~~~~~~~i~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ set_route

tripleping()
{
    local addr=$1
    local count_addr=$2
    local eth=$3
    local count_ping=3
    local res=0
    while ((${count_ping})); do
	${CMDPING} ${addr} > /dev/null
	res=$?
	if [ $res -eq 0 ]; then
	    break;
	fi
	((count_ping-=1))
    done

    echo -e "$count_addr\t$res\t$eth\t$addr" >> $TRIPLEPINGFILE
}

genconffile()
{
    if [ -f $CONFIG_FILE ]; then
	rm $CONFIG_FILE
    fi
    for ((i=0; i < ${#IPs[@]}; i++))
    do
	echo -e "${ETHs[i]} ${MACs[i]}" >> $CONFIG_FILE
    done
    echo -e "Configuration file created: $CONFIG_FILE\n"
}

readconffile()
{
	local CONF_ETHs=()
	local CONF_MACs=()

	if [ ! -f $CONFIG_FILE ]; then
		echo -e "\tWarning! Configuration file not found."
		echo -e "\tConfiguration check skipped .."
		exit 1
	else
		echo -e "Reading a configuration file: $CONFIG_FILE\n"
		while read STR; do
			CONF_ETHs+=($(echo $STR | awk '{print $1}'))
			CONF_MACs+=($(echo $STR | awk '{print $2}'))
		done < ${CONFIG_FILE}
	fi

	for ((i=0; i < ${#CONF_ETHs[@]}; ++i))
	do
		for ((j=0; j < ${#IPs[@]}; ++j))
		do
			if [[ "${CONF_ETHs[$i]}" == "${ETHs[$j]}" ]] &&
			   [[ "${CONF_MACs[$i]}" == "${MACs[$j]}" ]]; then
				INTERFACE_PRESENT=1
			fi
		done
		if [[ $INTERFACE_PRESENT != 1 ]]; then
			FAILED=1
			printf "%-15s   %-23s   [failed]\n" ${CONF_ETHs[$i]} ${CONF_MACs[$i]}
		else
			printf "%-15s   %-23s   [success]\n" ${CONF_ETHs[$i]} ${CONF_MACs[$i]}
			INTERFACE_PRESENT=
		fi
	done
	echo
	if [ ! -z $FAILED ]; then
		exit 1
	fi
}

up()
{
	res=0
	PRINT=1
	MINIMAL=0
	[[ $1 == "minimal" ]] && MINIMAL=1
	if [[ $1 == "silent" ]]; then
		PRINT=0;MINIMAL=0
	fi
	iptables-save > ${IPTABLES}
	ip route save scope link > ${IPROUTES}

	IPs=()
	MACs=()
	ETHs=()
	IPWs=()
	MCST_OS=0
	ifconfig -a |grep "inet addr" >/dev/null
	if [[ $? == 0 ]]; then
		MCST_OS=1
	fi

	o1=212
	o2=212
	o3=200
	o4=1

	unused_devs_list=`ls -l /sys/class/net/ | grep virtual | awk '{print $9}' | xargs | sed 's/ /\\\|/g'`
	unused_devs_list="${unused_devs_list}\|wlp"
	list=`ls /sys/class/net/ | grep -v "$unused_devs_list" | sort | tr '\n' ' '`
	ip_list=`ifconfig -a | grep "inet " | awk '{print $2}' | sed 's/#*addr://'`

	echo -e "$list\n"
	ip_index=0
	for e in $list; do
		if (($MCST_OS)); then
			ip=`ifconfig $e |grep "inet addr"`
			ip=${ip#*addr:}
			ip=${ip% Bcast*}
		else
			ip=`ifconfig $e |grep "inet " | awk '{print $2}'`
		fi

		if [[ -z $ip ]]; then
			if [[ "$e" == "eth"* ]]; then
				neth=$(echo $e | tr 'eth' ' ')
				((o=$o3+$neth))
			else
				while true; do
					((o=$o3+$ip_index))
					echo $ip_list | grep $o1.$o2.$o.$o4 > /dev/null
					if [[ "$?" == "1" ]]; then
						break;
					fi
					((ip_index++))
				done
			fi
			((ip_index++))
#			ip link set $e down
			ifconfig $e $o1.$o2.$o.$o4 up
			ip link set $e up
			ip addr add $o1.$o2.$o.$o4/24 broadcast 255.255.255.0 dev $e
			ifconfig $e $o1.$o2.$o.$o4 up

			ip="$o1.$o2.$o.$o4"
		fi
		if [[ $e == "eth0" ]]; then
			o1=$(echo $ip | awk -F. '{print $1}')
			o2=$(echo $ip | awk -F. '{print $2}')
			o3=$(echo $ip | awk -F. '{print $3}')
			o4=$(echo $ip | awk -F. '{print $4}')
		fi

	    mac=`cat /sys/class/net/${e}/address`

	    ETHs+=($e)
	    IPs+=($ip)
	    MACs+=($mac)
	done

	if [ ! -z $GENCONFFILE ]; then
		genconffile
	fi

	if [ ! -z $READCONFFILE ]; then
		readconffile
	fi

	c=1
	addr_count=0
	for ((i=0; i < (( ${#IPs[@]} - 1 )); ++i))
	do
		for ((j=$i + 1; j < ${#IPs[@]}; ++j))
		do
			left_addr="22.22.${c}.${i}"
			rght_addr="22.22.${c}.${j}"
			IPWs+=(${left_addr})
			IPWs+=(${rght_addr})
			set_route ${ETHs[i]} ${IPs[i]} ${MACs[i]} ${left_addr} ${ETHs[j]} ${IPs[j]} ${MACs[j]} ${rght_addr}
			tripleping ${left_addr} ${addr_count} ${ETHs[i]} &
			((addr_count++))
			tripleping ${rght_addr} ${addr_count} ${ETHs[j]} &
			((addr_count++))
			((c++))
		done
	done

	wait

	#tee command sometimes does not have time to write, so the file is empty
	#cat $TRIPLEPINGFILE | sort -k1n | tee ${TRIPLEPINGFILE}

	cat $TRIPLEPINGFILE | sort -k1n > ${TRIPLEPINGFILE}_
	mv ${TRIPLEPINGFILE}_ $TRIPLEPINGFILE

	addr_count=0
	addr_status_list=()
	while read STR; do
		addr_status_list[$addr_count]=$(echo $STR | awk '{print $2}')
		((addr_count++))
	done < ${TRIPLEPINGFILE}

	rm $TRIPLEPINGFILE

	addr_count=0

	c=1
	pair_count=0
	for ((i=0; i < (( ${#IPs[@]} - 1 )); ++i))
	do
		for ((j=$i + 1; j < ${#IPs[@]}; ++j))
		do
			ping_res=0
			left_addr="22.22.${c}.${i}"
			rght_addr="22.22.${c}.${j}"
			((ping_res+=${addr_status_list[$addr_count]}))
			((addr_count++))
			((ping_res+=${addr_status_list[$addr_count]}))
			((addr_count++))

			if [ ${ping_res} -eq 0 ]; then
				link="---"
				st="ok"
				(($MINIMAL)) && PRINT=1
				ffound=0
				for ((z=0; z < ${#eth_list[@]}; ++z)); do
					if [[ ${eth_list[z]} == ${ETHs[i]} ]] || [[ ${eth_list[z]} == ${ETHs[j]} ]]; then
						((ffound+=1))
					fi
				done
				if [[ $ffound == 0 ]]; then
					eth_list+=("${ETHs[i]}")
					eth_list+=("${ETHs[j]}")
					pair_list+=("${IPERF} -s -1 -B ${IPs[j]} -D; ${IPERF} -c ${rght_addr} -t $TESTTIME;")
					serv_list+=("${IPs[j]}")
					clnt_list+=("${rght_addr}")
					((pair_count+=1))
				fi
			else
				link="-x-"
				st="no link"
				(($MINIMAL)) && PRINT=0
			fi

			(($PRINT)) && printf "%s\n"  "${ETHs[i]} ${IPs[i]} ${left_addr} ${link} ${ETHs[j]} ${IPs[j]} ${rght_addr} ${st}"


			((c++))
		done
		if (($MINIMAL)) || (($SILENT)); then
			echo
		fi
	done

	if ! (($pair_count)); then
	    echo "ERROR! Pairs not found."
	    echo
	    echo "======================= ======================="
	    exit 1
	fi

	starttime=`date +%s`
	if (($PARALLEL)); then
	    echo "Attention! Parallel execution !"
	    echo
	    (( endtime = $starttime + $TESTTIME + $pair_count ))
	else
	    (( endtime = $starttime + ($TESTTIME*$pair_count) ))
	fi
	print_date $endtime

	printf "%s\n" "${IPWs[@]}" > ${ARP}
	#printf "%s\n" "${eth_list[@]}"

	echo
	echo "*** Found ${pair_count} unique pairs ***"
	echo
	printf "%s\n" "${pair_list[@]}"
	echo
	echo "*** Start servers ***"
	echo
	for i in ${serv_list[@]}; do
		echo "${IPERF} -s -1 -B $i -D"
		${IPERF} -s -1 -B $i -D
		sleep 1
	done

	echo
	echo "*** Start clients ***"
	echo
	cnt=0
	for i in ${clnt_list[@]}; do
		if (($PARALLEL)); then
		    echo "${IPERF} -c $i -t $TESTTIME ${IARGS} &"
		    ${IPERF} -c $i -t $TESTTIME ${IARGS} &
		    pids[${cnt}]=$!
		    ((cnt+=1))
		    sleep 1
		else
		    echo "${IPERF} -c $i -t $TESTTIME ${IARGS}"
		    ${IPERF} -c $i -t $TESTTIME ${IARGS}
		    if [[ $? != 0 ]]; then
			((res+=1))
		    fi
		fi
	done

	if (($PARALLEL)); then
	    for pid in ${pids[*]}; do
		wait $pid
	    done
	fi

	# lazy kill all iperf*
	killall iperf 2>/dev/null
	killall iperf3 2>/dev/null

	exit $res
}

down()
{
	[[ -f ${ARP} ]] || return

	ips=$(<${ARP})

	for i in $ips
	do
		arp -d $i
	done

	iptables-restore < ${IPTABLES}
	ip route flush scope link
	ip route restore < ${IPROUTES}
}

usage()
{ IFS=''; while read helpstr
do
	echo "$helpstr"
done << EOF
Setup network adapters and testing.

Usage: $0 [-t testtime] [-p] [-g | -r] [-f] | [-d] 
 - no args    -- configures adapters, print links and start test each pair at 60 seconds.
 -t VALUE     -- configures adapters, print links and start tests at VALUE seconds.
 -p           -- run all test in parallel (default sequential)
 -d           -- restores settings and terminate
 -g	      -- generate default configuration file
 -r	      -- read default configuration file
 -f	      -- user configuration file
 -h           -- help

Examples:
1. Generate default configuration file, then run testing for 10 seconds.
    $0 -t 10 -g
2. Read default configuration file, then run testing for 10 seconds.
    $0 -t 10 -r
3. Generate user configuration file, then run testing for 10 seconds.
    $0 -t 10 -g -f /home/user/eth_iperf.conf
4. Read user configuration file, then run testing for 10 seconds.
    $0 -t 10 -r -f /home/user/eth_iperf.conf
5. Generate and read default configuration file, then run testing for 10 seconds.
    $0 -t 10 -g -r
6. Generate and read user configuration file, then run testing for 10 seconds.
    $0 -t 10 -g -r -f /home/user/eth_iperf.conf
EOF
}

args=("$@")
if (( $# )); then
	if [[ $1 == $0 ]]; then
		shift
		args=("$@")
	fi
fi

OPTIONS="hdt:pgrf:"
while getopts $OPTIONS OPT
do
	case $OPT in
	d ) down;exit 0;;
	p ) PARALLEL=1;;
	t ) TESTTIME=${OPTARG};
		if ! [[ "${TESTTIME}" =~ ^[0-9]+$ ]]; then
			echo;
			echo "ERROR! Please, input input integer number !";
			echo;
			usage;exit 1;
		fi;;
	f ) CONFIG_FILE=${OPTARG};;
	g ) GENCONFFILE=1;;
	r ) READCONFFILE=1;;
	h ) usage;exit 0;;
	esac
done

echo "======================= ======================="
up # minimal
echo "======================= ======================="

exit 0
