#!/bin/bash

BASEDIR="/sys/bus/iio/devices"
DIR=
DEVNAME="icm20948"

if [[ ! -d ${BASEDIR} ]]; then
	echo "SKIP: No icm20948 directory found"
	exit 1
fi

LISTDIR=`ls -1 ${BASEDIR} | grep iio`
if [[ -z ${LISTDIR} ]]; then
	echo "SKIP: No icm20948 devices directory found"
	exit 1
fi

for DIR in ${LISTDIR}; do
	if [[ -d "${BASEDIR}/$DIR" ]] ; then
		if [[ "`cat ${BASEDIR}/$DIR/name`" == "$DEVNAME" ]] ; then
			echo "icm20948 directory found: ${BASEDIR}/${DIR}"
			echo
			break
		fi
	else
		echo "SKIP: No icm20948 devices in base directory found"
		exit 1
	fi
done
DIR="${BASEDIR}/${DIR}"


accel_x="`cat ${DIR}/in_accel_x_raw`"
accel_y="`cat ${DIR}/in_accel_y_raw`"
accel_z="`cat ${DIR}/in_accel_z_raw`"
accel_scale="`cat ${DIR}/in_accel_scale`"

a_x=$(bc<<<$accel_scale*$accel_x)
a_y=$(bc<<<$accel_scale*$accel_y)
a_z=$(bc<<<$accel_scale*$accel_z)

ax=$(bc<<<$a_x*2/1+20)
ay=$(bc<<<$a_y*2/1+20)
az=$(bc<<<$a_z*2/1+20)
pax="printf '#%.0s' {1..${ax%.*}}"
pay="printf '#%.0s' {1..${ay%.*}}"
paz="printf '#%.0s' {1..${az%.*}}"

printf "accel x = %-6.2f m/s^2  " $a_x; eval $pax; printf " \n"
printf "accel y = %-6.2f m/s^2  " $a_y; eval $pay; printf " \n"
printf "accel z = %-6.2f m/s^2  " $a_z; eval $paz; printf " \n"
echo


angl_x="`cat ${DIR}/in_anglvel_x_raw`"
angl_y="`cat ${DIR}/in_anglvel_y_raw`"
angl_z="`cat ${DIR}/in_anglvel_z_raw`"
angl_scale="`cat ${DIR}/in_anglvel_scale`"

an_x=$(bc<<<$angl_scale*$angl_x)
an_y=$(bc<<<$angl_scale*$angl_y)
an_z=$(bc<<<$angl_scale*$angl_z)

anx=$(bc<<<$an_x*10/1)
any=$(bc<<<$an_y*10/1)
anz=$(bc<<<$an_z*10/1)
panx="printf '#%.0s' {1..${anx%.*}}"
pany="printf '#%.0s' {1..${any%.*}}"
panz="printf '#%.0s' {1..${anz%.*}}"

printf "anglv x = %-6.2f Rad/s  " $an_x; eval $panx; printf " \n"
printf "anglv y = %-6.2f Rad/s  " $an_y; eval $pany; printf " \n"
printf "anglv z = %-6.2f Rad/s  " $an_z; eval $panz; printf " \n"
echo



exit 0


# Directory entries:
#
# in_accel_filter_low_pass_3db_frequency
# in_accel_filter_low_pass_3db_frequency_available
# in_accel_scale
# in_accel_scale_available
# in_accel_x_calibbias
# in_accel_x_raw
# in_accel_y_calibbias
# in_accel_y_raw
# in_accel_z_calibbias
# in_accel_z_raw
# in_anglvel_filter_low_pass_3db_frequency
# in_anglvel_filter_low_pass_3db_frequency_available
# in_anglvel_scale
# in_anglvel_scale_available
# in_anglvel_x_calibbias
# in_anglvel_x_raw
# in_anglvel_y_calibbias
# in_anglvel_y_raw
# in_anglvel_z_calibbias
# in_anglvel_z_raw
# in_magn_scale
# in_magn_x_raw
# in_magn_y_raw
# in_magn_z_raw
# in_mount_matrix
# in_temp_filter_low_pass_3db_frequency
# in_temp_filter_low_pass_3db_frequency_available
# in_temp_offset
# in_temp_raw
# in_temp_scale
# name
