#!/bin/bash

PMC_VER_FILE=`find /sys/devices/system/ -name "ver_input"`
if [[ ! -z $PMC_VER_FILE ]]; then
	echo "== Check PMC version"
	PMC_VERSION=`cat $PMC_VER_FILE`
	echo -e "\tPMC version: $PMC_VERSION"
	exit 0
fi

PARAM_REGS=(0x9010 0x9014 0x9018 0x901c)
PARAM_NAMES=(PARAM_0 PARAM_1 PARAM_2 PARAM_3)
DOORBEL_REG=0x9024
srv_get_fw_revision=0x10000
srv_get_deadbeef=0x18000
srv_deadbeef_val=(0xdeadbeef 0xdeadc0de 0xdeadcafe 0x0)

NO_SIC_REGS=1
if ( [ -f /proc/sicread ] && [ -f /proc/sicregs ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicregs
	Path_SICW=/proc/sicwrite
	NO_SIC_REGS=0
elif  ( [ -f /proc/sicread ] && [ -f /proc/sicwrite ] ); then
	Path_SICw=/proc/sicread
	Path_SICr=/proc/sicread
	Path_SICW=/proc/sicwrite
	NO_SIC_REGS=0
fi

if (($NO_SIC_REGS)); then
	uname -a
	echo -e "\nError: Not found kernel interface for PMC !\n"
	exit 1
fi

(( NODE = `cat $Path_SICr | wc -l` - 1 ))

get_fw_revision()
{
	echo "== Check PMC version"
	local node=0
	echo "$node $DOORBEL_REG $srv_get_fw_revision" > $Path_SICW
	echo ${PARAM_REGS[0]} > $Path_SICw
	FSTR=`cat ${Path_SICr} | grep "node: $node reg"`
	reg=(`echo ${FSTR,,} | awk '{print $4}' | tr '():' ' '`)
	reg_val=(`echo ${FSTR,,} | awk '{print $5}'`)
	if [[ ${PARAM_REGS[0]} == $reg ]] && [[ ! -z $reg_val ]]; then
		version_patch=$(( ($reg_val & 0x000000ff) >> 0 ))
		version_minor=$(( ($reg_val & 0x0000ff00) >> 8 ))
		version_major=$(( ($reg_val & 0x00ff0000) >> 16 ))
		echo -e "\tPMC version: $version_major.$version_minor.$version_patch"
	fi
}

get_deadbeef()
{
	local err=
	echo "== Check get_deadbeef service"
	for cur_node in `seq 0 $NODE`; do
		echo "$cur_node $DOORBEL_REG $srv_get_deadbeef" > $Path_SICW
		for i in ${!PARAM_REGS[@]}; do
			echo -e "\tPMC register: ${PARAM_NAMES[${i}]}"
			echo ${PARAM_REGS[${i}]} > $Path_SICw
			FSTR=`cat ${Path_SICr} | grep "node: $NODE reg"`
			reg=(`echo ${FSTR,,} | awk '{print $4}' | tr '():' ' '`)
			reg_val=(`echo ${FSTR,,} | awk '{print $5}'`)
			if [[ ${PARAM_REGS[${i}]} != $reg ]] || [[ ${srv_deadbeef_val[${i}]} != $reg_val ]]; then
				echo -e "\t\t$FSTR <-- ERROR"
				err=1
			else
				echo -e "\t\t$FSTR <-- SUCCESS"
			fi
			sleep .1
		done
	done
	if [[ ! -z $err ]]; then
		echo -e "\n\tError get_deadbeef service\n\tPMC test failed!"
		exit 1
	fi
}

get_deadbeef
get_fw_revision

exit 0
