#!/bin/bash

# Directory entries:
#
# in_accel_filter_low_pass_3db_frequency
# in_accel_filter_low_pass_3db_frequency_available
# in_accel_scale
# in_accel_scale_available
# in_accel_x_calibbias
# in_accel_x_raw
# in_accel_y_calibbias
# in_accel_y_raw
# in_accel_z_calibbias
# in_accel_z_raw
# in_anglvel_filter_low_pass_3db_frequency
# in_anglvel_filter_low_pass_3db_frequency_available
# in_anglvel_scale
# in_anglvel_scale_available
# in_anglvel_x_calibbias
# in_anglvel_x_raw
# in_anglvel_y_calibbias
# in_anglvel_y_raw
# in_anglvel_z_calibbias
# in_anglvel_z_raw
# in_magn_scale
# in_magn_x_raw
# in_magn_y_raw
# in_magn_z_raw
# in_mount_matrix
# in_temp_filter_low_pass_3db_frequency
# in_temp_filter_low_pass_3db_frequency_available
# in_temp_offset
# in_temp_raw
# in_temp_scale
# name
#
DIR=
DEVNAME="icm20948"

for((i=0;i<5;i++))
do
	DIR=/sys/bus/iio/devices/iio:device$i
	if [[ -d "$DIR" ]] ; then
		if [[ "`cat $DIR/name`" == "$DEVNAME" ]] ; then
			echo
			echo "icm20948 directory found: ${DIR}"
			echo
			break
		else
			continue
		fi
	else
		echo "No icm20948 directory found."
		exit 1
	fi
done


temp_raw="`cat ${DIR}/in_temp_raw`"
temp_off="`cat ${DIR}/in_temp_offset`"
temp_scale="`cat ${DIR}/in_temp_scale`"
echo "temp_raw=$temp_raw"
echo "temp_off=$temp_off"
echo "temp_scale=$temp_scale"
bc <<- EOF
	scale=10
	temp=(($temp_scale*$temp_raw)+$temp_off)/ 1000.
	print "\nT = ", temp, " C\n\n"
EOF

accel_x="`cat ${DIR}/in_accel_x_raw`"
accel_y="`cat ${DIR}/in_accel_y_raw`"
accel_z="`cat ${DIR}/in_accel_z_raw`"
accel_scale="`cat ${DIR}/in_accel_scale`"
echo "accel_x=$accel_x"
echo "accel_y=$accel_y"
echo "accel_z=$accel_z"
echo "accel_scale=$accel_scale"
bc <<- EOF1
	scale=10
	a_x=$accel_scale*$accel_x
	a_y=$accel_scale*$accel_y
	a_z=$accel_scale*$accel_z
	print "\naccel: x=", a_x, " y=", a_y, " z=", a_z, " (meter/sec^2)\n\n"
EOF1

angl_x="`cat ${DIR}/in_anglvel_x_raw`"
angl_y="`cat ${DIR}/in_anglvel_y_raw`"
angl_z="`cat ${DIR}/in_anglvel_z_raw`"
angl_scale="`cat ${DIR}/in_anglvel_scale`"
echo "angl_x=$angl_x"
echo "angl_y=$angl_y"
echo "angl_z=$angl_z"
echo "angl_scale=$angl_scale"
bc <<- EOF2
	scale = 10
	an_x=$angl_scale*$angl_x
	an_y=$angl_scale*$angl_y
	an_z=$angl_scale*$angl_z
	print "\nanglvel: x=", an_x, " y=", an_y, " z=", an_z, " (Radian/sec)\n\n"
EOF2

exit 0
