#!/bin/bash

ret=0


i2cdetect=`command -v i2cdetect`
if [ ! -f "$i2cdetect" ]; then
        echo "ERROR: please, install i2c-tools"
        exit 1
fi

sensorscmd=`command -v sensors`
if [ ! -f "$sensorscmd" ]; then
        echo "ERROR: please, install lm-sensors"
        exit 1
fi


### find i2c devices on devtree and i2c bus

chkdev () {
	busnum=${2##*-}
	devaddr=${1##*@}
	fulladdr=`printf "%s-%04x" ${busnum} "0x${devaddr}"`
	msg1="${2}/${fulladdr}"
	msg2=""
	if [ -d "${2}/${fulladdr}" ]; then
		#msg2=" - found"
		regs=`${i2cdetect} -y -a ${busnum} "0x${devaddr}" "0x${devaddr}" | grep UU`
		if [ -z "${regs}" ]; then
			msg2=" - found, NOT PRESENT"
			(( ret=ret+1 ))
		#else
		#	echo " - registered"
		fi
	else
		msg2=" - NOT FOUND"
		(( ret=ret+1 ))
	fi
	printf "    %-40s %s\n" "$msg1" "$msg2"
}


i2cpath="/sys/bus/i2c/devices"

findi2c () {
	#echo -n "    ${3}: "
	for i in `ls ${i2cpath} | grep i2c-`; do
		if [ -L ${i2cpath}/${i}/of_node ]; then
			rl=`readlink -e ${i2cpath}/${i}/of_node`
			if [ "${rl}" == "${1}/${2}" ]; then
				chkdev ${3} ${i2cpath}/${i}
			fi
		fi
	done
}


parsedev () {
	for d in `ls ${1}/${2} | grep @`; do
		compat=$(cat ${1}/${2}/${d}/compatible | tr '\0' '\n')
		if [ -f ${1}/${2}/${d}/label ]; then
			label=$(cat ${1}/${2}/${d}/label | tr '\0' '\n')
		else
			label=""
		fi
		echo " -- dtb-name: ${label}"
		echo "    ${2}/${d}: ${compat}"
		findi2c ${1} ${2} ${d}
		parsebus ${1}/${2} ${d}
	done
}


parsebus () {
	for b in `ls ${1}/${2} | grep i2c@`; do
		echo "=== ${2}/${b} bus"
		stat=""
		if [ -f ${1}/${2}/${b}/status ]; then
			stat=$(cat ${1}/${2}/${b}/status | tr '\0' '\n')
		fi
		if [ "${stat}" == 'disabled' ]; then
			echo "bus disabled"
		else
			parsedev ${1}/${2} ${b}
		fi
	done
}


basepath="/sys/firmware/devicetree/base"

echo ""
echo -e "\tI2C DEVICES"
echo ""

for db in `ls ${basepath} | grep l_i2c@`; do
	echo "*** ${db} domain"
	parsebus ${basepath} ${db}
done


### find sensors

echo ""
echo -e "\tSENSORS"
echo ""

# Get Adapter list
prevline=""
adapters=""
while IFS= read -r line
do
	if [[ "$line" == "Adapter:"* ]]; then
		adapters+="${prevline};"
	fi
	prevline=$line
done < <($sensorscmd)

myifs=$IFS
IFS=$';'
i=1
for adapter in $adapters
do
	comment=""
	if [[ "$adapter" == *"-i2c-"* ]]; then
		busaddr=`echo "$adapter" | awk -F"-" '{print "i2c-" $3 "/" $3 "-00" $4}'`
		# get hwmon name"
		IFS=$myifs
		for hwmon in `ls -1 /sys/class/hwmon`
		do
			slink=`realpath /sys/class/hwmon/$hwmon`
			if [[ "$slink" == *"$busaddr"* ]]; then
				comment=" - ${hwmon}"
				if [ -f "/sys/class/hwmon/${hwmon}/of_node/label" ]; then
					comment+=": "
					comment+=$(cat /sys/class/hwmon/${hwmon}/of_node/label | tr '\0' '\n')
				fi
			fi
		done
	fi
	echo "$i. ${adapter}${comment}"
	i=$(( $i + 1 ))
done


### exit

echo ""
if [ $ret -eq 0 ]; then
	echo "SUCCESS"
	exit 0
else
	echo "FAIL (${ret} sensors not found)"
	exit 1
fi

