#!/bin/bash

umask 002

DIR=$(dirname `realpath $0`)

ALL_FOUND=0
DETECT=0

PARAM_FILE="${DIR}/param"
ERR=0

OPTIONS="hadf:"
while getopts $OPTIONS OPT
do
	case $OPT in
	a ) ALL_FOUND=1;;
	d ) DETECT=1;;
 	f ) PARAM_FILE=${OPTARG};;
	h ) echo "
 Usage: $0 [-a] | [-d] | [-f param file]
	-a      - search all i2c devices and attempt to dump to screen
	-d      - only detect and create param file for check i2c
	-f file - set name for param file with check list devices
";exit 0;;
	esac
done

if [[ -z `command -v i2cdetect` ]]; then
    echo "ERROR: i2cdetect utils not found"
    exit 1
fi

modprobe i2c-dev
modprobe pmbus

I2CDETECT="i2cdetect -a -q "

#only detect and/or dump all find and not busy i2c
detect_i2c_bus() {
    #buslist=`seq 0 128`
    buslist=`i2cdetect -l | grep l_i2c | awk '{print $1}' | awk -F- '{print  $2}'`
    for b in $buslist; do
	if [[ ! -z $1 ]]; then
	    echo " --- I2C bus: $b ------------"
	fi
	rawbus=$(${I2CDETECT} -y $b 2>/dev/null)
	if [[ $? != 0 ]]; then
	    if [[ ! -z $1 ]]; then
		echo " --- End of I2C buses ..."
	    fi
	    break
	fi
	businfo=$(echo $rawbus | sed 's/0 1 2 3 4 5 6 7 8 9 a b c d e f//g' | sed 's/[0-9][0-9]://g')

	C=0
	devlist=""
	for i in $businfo; do
	    if [[ $i != "--" ]]; then
		resdetect=1
		if [[ ! -z $1 ]]; then
		    printf '\nFound: bus: %d, Device address: %s [0x%02x %02d]\n' $b $i $C $C
		    if [[ $i != "UU" ]]; then
			i2cdump -y $b $C W
			i2cdump -y $b $C W | grep -v "0123456789abcdef" | cut -b 5-51 | sed "s/[Xf ]//g" | grep -v "^$" >/dev/null
			resdetect=$?
		    else
			echo "For busy devices - dump are not run."
			resdetect=0
		    fi
		else
		    if [[ $i != "UU" ]]; then
			i2cdump -y $b $C W | grep -v "0123456789abcdef" | cut -b 5-51 | sed "s/[Xf ]//g" | grep -v "^$" >/dev/null
			resdetect=$?
		    else
			resdetect=0
		    fi
		fi
		if [[ ${resdetect} -eq 0 ]]; then
			devlist="$devlist `printf '%02x' $C`"
		fi
	    fi
	    (( C = $C + 1 ))
	done
	echo "$b : $devlist"
    done
    echo
}

check_with_file() {
    if [[ ! -f $1 ]]; then
	echo "File with check params not found: \"${1}\"."
	echo "Trying to create params file: \"${1}\"."
	generate_params_file
	    if [[ ! -f $1 ]]; then
		echo "Params file not created."
		ERR=1
		return
	    fi
    fi

    while read line ; do
	b=$(echo $line | awk -F: '{print $1}')
	checklist=$(echo $line | awk -F: '{print $2}')

	echo " --- I2C bus: $b ------------"
	rawbus=$(${I2CDETECT} -y $b 2>/dev/null)
	if [[ $? != 0 ]]; then
	    echo "I2C bus $b - not found"
	    (( ERR = $ERR + 1 ))
	    continue
	fi
	businfo=$(echo $rawbus | sed 's/0 1 2 3 4 5 6 7 8 9 a b c d e f//g' | sed 's/[0-9][0-9]://g')

	C=0
	devlist=""
	for i in $businfo; do
	    if [[ $i != "--" ]]; then
		resdetect=1
		if [[ $i != "UU" ]]; then
			i2cdump -y $b $C W | grep -v "0123456789abcdef" | cut -b 5-51 | sed "s/[Xf ]//g" | grep -v "^$" >/dev/null
			resdetect=$?
		else
			resdetect=0
		fi
		if [[ ${resdetect} -eq 0 ]]; then
			devlist="$devlist `printf '%02x' $C`"
		fi
	    fi
	    (( C = $C + 1 ))
	done

	echo "On board: $devlist"
	echo "Checking: $checklist"

	for i in $checklist; do
	    echo $devlist | grep $i 2>/dev/null >/dev/null
	    if [[ $? != 0 ]]; then
		echo "Device: $i not found"
		(( ERR = $ERR + 1 ))
	    fi
	done
    done < <(cat ${PARAM_FILE} | grep -v ^# | sed '/^[[:space:]]*$/d')
}

generate_params_file() {
    cat << 'EOF' > ${PARAM_FILE}
###
# Auto-generating file
# You must fill check params with next rules:
# i2c-bus-number : device numbers on that i2c bus
#
# Example:
# 0 : 0c 18 1a 1c 1e 2e
# Test completed if we found all point i2c buses and all
# devices from that file
###

EOF
    detect_i2c_bus >> ${PARAM_FILE}
    cat ${PARAM_FILE}
    echo "Create file: ${PARAM_FILE}"
}

if ((${DETECT})); then
    echo "Test is NOT run !!!"
    generate_params_file
    exit 0
fi

if ((${ALL_FOUND})); then
    detect_i2c_bus dump
else
    echo "Check from file: ${PARAM_FILE}"
    check_with_file $PARAM_FILE
fi

if (($ERR)); then
    echo "Test: ERROR"
else
    echo "Test: SUCCESS"
fi

exit $ERR
